@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_timestamp.c,v 1.20 2011/09/05 09:49:37 kitagawa Exp $
+ * $Header$
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2011	PgPool Global Development Group
+ * Copyright (c) 2003-2013	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -314,12 +314,13 @@ get_current_timestamp(POOL_CONNECTION_POOL *backend)
 {
 	POOL_SELECT_RESULT *res;
 	POOL_STATUS		 status;
-	static char		timestamp[32];
+	static char		timestamp[64];
 
 	status = do_query(MASTER(backend), "SELECT now()", &res, MAJOR(backend));
 	if (status != POOL_CONTINUE)
 	{
-		pool_error("get_current_timestamp: do_query faild");
+		pool_error("get_current_timestamp: do_query failed");
+		free_select_result(res);
 		return NULL;
 	}
 
@@ -870,7 +871,8 @@ static A_Const *makeStringConstFromQuery(POOL_CONNECTION_POOL *backend, char *ex
 	status = do_query(MASTER(backend), query, &res, MAJOR(backend));
 	if (status != POOL_CONTINUE)
 	{
-		pool_error("makeStringConstFromQuery: do_query faild");
+		pool_error("makeStringConstFromQuery: do_query failed");
+		free_select_result(res);
 		return NULL;
 	}
 
